import streamlit as st
from os import environ
from langchain.chat_models import ChatOpenAI
from langchain.embeddings import OpenAIEmbeddings
from langchain.embeddings import SentenceTransformerEmbeddings
from langchain.llms.huggingface_pipeline import HuggingFacePipeline
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, pipeline
from enum import Enum

from src.settings import template

openai_key = environ.get("OPENAI_API_KEY")


class LLMProvider(Enum):
    Local = 1
    OpenAI = 2


def load_llm(llm_provider: LLMProvider):
    if llm_provider == LLMProvider.Local:
        return load_local()
    elif llm_provider == LLMProvider.OpenAI:
        return load_openai()


@st.cache_resource
def load_local():
    print("Loading Local LLM...")
    model_name = "google/flan-t5-large"
    tokenizer = AutoTokenizer.from_pretrained(model_name)
    model = AutoModelForSeq2SeqLM.from_pretrained(model_name)
    pipe = pipeline(
        "text2text-generation",
        model=model,
        tokenizer=tokenizer,
        max_length=512,
        temperature=0,
        top_p=0.95,
        repetition_penalty=1.15
    )
    llm = HuggingFacePipeline(pipeline=pipe)
    embeddings = SentenceTransformerEmbeddings(model_name="all-MiniLM-L6-v2")

    return llm, embeddings


@st.cache_resource
def load_openai():
    print("Loading OpenAI LLM...")
    llm = ChatOpenAI(openai_api_key=openai_key)
    embeddings = OpenAIEmbeddings(openai_api_key=openai_key)
    return llm, embeddings